/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile: XSDValidatorHandler.java,v $
 * Version:       $Name:  $ $Revision: 1.3 $
 * Last Modified: $Date: 2004/06/21 10:25:42 $
 *
 * CHANGELOG
 * [30/9/2011 maw] Added the use of a CatalogResolver.
 *****************************************************************************/
package org.ten60.netkernel.xml.accessor;

import org.ten60.netkernel.xml.xda.*;
import org.ten60.netkernel.xml.xahelper.*;
import org.ten60.netkernel.xml.util.*;
import com.ten60.netkernel.util.SysLogger;


import org.xml.sax.helpers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.io.*;
import org.apache.xml.resolver.tools.CatalogResolver;

/**
 * An XSD Validator Handler for catching errors and warnings from XSD validation
 * @author  pjrhplb
 */
public class XSDValidatorHandler extends DefaultHandler
{
    private int errorcount;
    private int warningcount;
    private Document results=org.ten60.netkernel.xml.util.XMLUtils.getInstance().newDocument();
    private DOMXDA wrt=new DOMXDA(results, null);
    private XAHelper mHelper;
    private String mSchemaURI;
    private IXDAReadOnly mSchema;
    private CatalogResolver mResolver;
    
    /** Creates a new instance of XSDValidatorContentHandler */
    public XSDValidatorHandler(XAHelper aHelper, String aSchemaURI, IXDAReadOnly aSchema)
    {   mHelper=aHelper;
	    mResolver = new CatalogResolver();
        mSchemaURI = aSchemaURI;
        mSchema=aSchema;
        try
        {	wrt.appendPath("/","/b","f");
            wrt.appendPath("/b","/errors","");
            wrt.appendPath("/b","/warnings","");
        }
        catch(Exception e)
        {	e.printStackTrace();
        }
    }
	
    /*Receive notification of a recoverable parser error.*/
    public void error(SAXParseException e)
    {   try
        {	wrt.appendPath("/b/errors","/error",e.getMessage());
        }
        catch(Exception locale)
        {	locale.printStackTrace();
        }
        errorcount++;
    }
    
    /*Report a fatal XML parsing error.*/
    public void fatalError(SAXParseException e)
    {   try
        {	wrt.appendPath("/b/errors","/error",e.getMessage());
        }
        catch(Exception locale)
        {	locale.printStackTrace();
        }
        errorcount++;
    }
    /*     Receive notification of a parser warning.    */
    public void warning(SAXParseException e)
    {   try
        {	wrt.appendPath("/b/warnings","/warning",e.getMessage());
        }
        catch(Exception locale)
        {	locale.printStackTrace();
        }
        warningcount++;
    }
    
    /** @return true if document is valid */
    public boolean isvalid()
    {   if(errorcount>0) return false;
        else return true;
    }
    
    /** @return the number of warnings */
    public int getWarnings()
    {	return warningcount;
    }
    
    /** @return the number of errors */
    public int getErrors()
    {	return warningcount;
    }
    
    /** @return a cannonical boolean document */
    public Document getDocument()
    {   try
        {	if(errorcount>0)
            {	wrt.setText("/b","f");
            }
            else wrt.setText("/b","t");
            return wrt.toDOM();
        }
        catch(Exception e)
        {	e.printStackTrace();
        }
        return null;
    }
    
    public InputSource resolveEntity(String publicId, String systemId)
        throws SAXException
    {   try
        {   InputSource result;
            if (systemId.equals(mSchemaURI))
            {   StringWriter sw=new StringWriter();
				mSchema.serialize(sw,false);
				StringReader reader = new StringReader( sw.toString() );
                result = new InputSource(reader);
            }
            else
            {  result = mResolver.resolveEntity(publicId,systemId);
               if (result == null)
	       {  result = mHelper.resolveEntity(publicId,systemId);
	       }
            }
            return result;
        } catch (Exception e)
        {   return null;
        }
    }
}
